package classes;

import java.util.HashMap;
import java.util.Map;

/*
 * This class would be used to keep a list of accidentals used in a measure,
 * to see if a new one can be omitted (previously used in the measure) or it
 * must be drawn.
 * 
 * XML will need this for a correct use of the <accidental> tag, or else it
 * will unnecessarily add an accidental to all altered notes.
 * 
 */
public class KeyAccidentals {
	
	Map<String, Integer> keySignature = new HashMap<String, Integer>();
	Map<String, Integer> keyMeasure = new HashMap<String, Integer>();
	
	int mode; // 0 is major, 1 is minor
	
	public KeyAccidentals (int fifths, int keyQuality)
	{
		
		if ((fifths >= -7 && fifths <= 7) || (keyQuality < 0 || keyQuality > 1))
		{
			mode = keyQuality;
			
			resetSignature();
			
			setSignatureAccs(fifths);
		
		} else {
			// invalid data
		}
		
		resetMeasure();
	}

	public KeyAccidentals (String key, int keyQuality)
	{
		mode = keyQuality;
		
		if (key.equalsIgnoreCase("C") || key.equalsIgnoreCase("Am"))
		{ setSignatureAccs(0); }
		else if (key.equalsIgnoreCase("G") || key.equalsIgnoreCase("Em"))
		{ setSignatureAccs(1); }
		else if (key.equalsIgnoreCase("D") || key.equalsIgnoreCase("Bm"))
		{ setSignatureAccs(2); }
		else if (key.equalsIgnoreCase("A") || key.equalsIgnoreCase("F#m"))
		{ setSignatureAccs(3); }
		else if (key.equalsIgnoreCase("E") || key.equalsIgnoreCase("C#m"))
		{ setSignatureAccs(4); }
		else if (key.equalsIgnoreCase("B") || key.equalsIgnoreCase("G#m"))
		{ setSignatureAccs(5); }
		else if (key.equalsIgnoreCase("F#") || key.equalsIgnoreCase("D#m"))
		{ setSignatureAccs(6); }
		else if (key.equalsIgnoreCase("C#") || key.equalsIgnoreCase("A#m"))
		{ setSignatureAccs(7); }
		else if (key.equalsIgnoreCase("F") || key.equalsIgnoreCase("Dm"))
		{ setSignatureAccs(-1); }
		else if (key.equalsIgnoreCase("Bb") || key.equalsIgnoreCase("Gm"))
		{ setSignatureAccs(-2); }
		else if (key.equalsIgnoreCase("Eb") || key.equalsIgnoreCase("Cm"))
		{ setSignatureAccs(-3); }
		else if (key.equalsIgnoreCase("Ab") || key.equalsIgnoreCase("Fm"))
		{ setSignatureAccs(-4); }
		else if (key.equalsIgnoreCase("Db") || key.equalsIgnoreCase("Bbm"))
		{ setSignatureAccs(-5); }
		else if (key.equalsIgnoreCase("Gb") || key.equalsIgnoreCase("Ebm"))
		{ setSignatureAccs(-6); }
		else if (key.equalsIgnoreCase("Cb") || key.equalsIgnoreCase("Abm"))
		{ setSignatureAccs(-7); }
	
	}
	
	private void setSignatureAccs(int fifths) {
		if (fifths > 0)
		{
			// G, Em
			keySignature.put("f", 1);
			if (fifths > 1)
			{
				// D, Bm
				keySignature.put("c", 1);
				if (fifths > 2)
				{
					// A, F#m
					keySignature.put("g", 1);
					if (fifths > 3)
					{
						// E, C#m
						keySignature.put("d", 1);
						if (fifths > 4)
						{
							// B, G#m
							keySignature.put("a", 1);
							if (fifths > 5)
							{
								// F#, D#m
								keySignature.put("e", 1);
								if (fifths > 6)
								{
									// C#, A#m
									keySignature.put("b", 1);
								}
							}
						}
					}
				}
			}
		}
		else if (fifths < 0)
		{
			// F, Dm
			keySignature.put("b", -1);
			if (fifths < -1)
			{
				// Bb, Gm
				keySignature.put("e", -1);
				if (fifths < -2)
				{
					// Eb, Cm
					keySignature.put("a", -1);
					if (fifths < -3)
					{
						// Ab, Fm
						keySignature.put("d", -1);
						if (fifths < -4)
						{
							// Db, Bbm
							keySignature.put("g", -1);
							if (fifths < -5)
							{
								// Gb, Ebm
								keySignature.put("c", -1);
								if (fifths < -6)
								{
									// Cb, Abm
									keySignature.put("f", -1);
								}
							}
						}
					}
				}
			}
		}
	}
	
	private void resetSignature()
	{
		keySignature.put("a", 0);
		keySignature.put("b", 0);
		keySignature.put("c", 0);
		keySignature.put("d", 0);
		keySignature.put("e", 0);
		keySignature.put("f", 0);
		keySignature.put("g", 0);
	}
	
	public void resetMeasure()
	{
		keyMeasure = new HashMap<String, Integer>(keySignature);
	}
	
	public int getSignatureAcc(String pitch)
	{
		return keySignature.get(pitch);
	}
	
	public int getMeasureAcc(String pitch)
	{
		return keyMeasure.get(pitch);
	}
	
	public void setAcc(String pitch, int acc)
	{
		keyMeasure.put(pitch, acc);
	}
}
