/*

<This Java Class is part of the jMusic API version 1.5, March 2004.>

Copyright (C) 2000 Andrew Brown and Andrew Sorensen

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or any
later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/ 

package gui;

import java.awt.event.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.awt.*; 

import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.xml.stream.XMLStreamException;

import jm.gui.cpn.BassStave;
import jm.gui.cpn.CpnZoomScreen;
import jm.gui.cpn.GrandStave;
import jm.gui.cpn.JmMidiPlayer;
import jm.gui.cpn.LetterNotesEditor;
import jm.gui.cpn.ParmScreen;
import jm.gui.cpn.PhraseViewer;
import jm.gui.cpn.PianoStave;
import jm.gui.cpn.Stave;
import jm.gui.cpn.TrebleStave;
import jm.music.data.*;
import jm.util.Read;
import jm.util.View;
import jm.util.Write;
import jm.util.Play;
import jm.JMC;
import classes.ScoreInformation;
import classes.WizardData;
import exceptions.StAXParsingError;

/**
* This class displays a frame with a common practice notation display
* of the score passed to it.
* The parameter and add data by text attributes only work on the first stave.
* Some GPL changes for jMusic CPN Written by Al Christians 
* (achrist@easystreet.com).
* Contributed by Trillium Resources Corporation, Oregon's
* leading provider of unvarnished software.
* @author Andrew Brown
*/

@SuppressWarnings("serial")
public class Notate extends Frame implements 
                ActionListener, 
                WindowListener, JMC {
	private JFrame frame;
	
	static ScoreInformation si = ScoreInformation.getSI();
	private Score score = si.getScore();
	//GUI_CreateScore cs = new GUI_CreateScore();
	Part part = new Part();
	Phrase phrase = new Phrase();
	
	
   // private Score score;
    //private Stave stave;
    //private Phrase phrase;
    private Phrase[] phraseArray;
    private Stave[] staveArray;
    private int locationX = 0, locationY = 0;
    private Dialog keyDialog, timeDialog;
    private MenuItem keySig, open, openJmXml, openjm, delete, clear, 
                    newStave, close, timeSig, saveJmXml, saveJM, saveMidi, quit,
                    trebleStave, bassStave, pianoStave, grandStave, automaticStave,
                    
                    setParameters,
                    playAll,
                    playMeasure,
                    repeatAll,
                    repeatMeasure,
                    stopPlay,
                    addNotes,
                    adjustTiming, viewDetails, viewTitle,
                    viewZoom, barNumbers,
                    expMXML, expNewMXML, impMXML, expLy, newWizard, welcome;
    				
    public boolean timeToStop;                 				
    // the panel for all the stave panels to go in to
    private Panel scoreBG;
    // the constraints for the scoreBG layout
    private GridBagConstraints constraints;
    private GridBagLayout layout;
    // The scoreBg goes into this scroll pane to enable navigation
    private ScrollPane scroll;

    private String lastFileName   = "*.mid";
    private String lastDirectory  = "";
    private boolean     zoomed;
    private Phrase      beforeZoom = new Phrase();
    private Phrase      afterZoom = new Phrase();
    /* The height of the notate window */
    private int height = 0;
    private int width = 700;

   
    
    public Notate(int locX, int locY) {
        this(new Phrase(), locX, locY);
        clearZoom();
    }
    
    public Notate(Phrase afterZoom2) {
        this(0, 0);
        clearZoom();
    }
    
    
    public Notate(WizardData wd) {
    	
    	/*
		If wd.getNote() is NULL, we're importing the score from an XML file. If so, divisions is already set.
		If wd.getNote() is not NULL, we're importing the score from GUI_CreateScore.
		*/
    	
    	if (wd.getNote() != null)
    	{
    		score = new Score();
    	}

    	// Composer
    	si.setComposer(wd.getComposer());
    	
    	// Title
    	score.setTitle(wd.getTitle());
    	
    	// Tempo
    	if (wd.getTempo() == null)
    	{
    		score.setTempo(120.0);
    	}
    	else
    	{
    		score.setTempo(Double.parseDouble(wd.getTempo()));
    	}
    	
    	// Clef (1)
    	String clef = wd.getClef();
		
    	// Time signature
    	score.setNumerator((int) Double.parseDouble(wd.getNumerator()));
    	//score.setDenominator((int) Double.parseDouble(wd.getDenominator()));
    	
    	if (wd.getNote() == null)
    	{
    		// Key quality
    		score.setKeySignature(Integer.parseInt(wd.getKeySignature()));
    		
    		// Key signature
    		if (wd.getKeyQuality() == null)
    		{
    			score.setKeyQuality(0);
    		}
    		else if (wd.getKeyQuality().equals("major"))
    		{
    			score.setKeyQuality(0);
    		}
    		else if (wd.getKeyQuality().equals("minor"))
    		{
    			score.setKeyQuality(1);
    		}
    	}
    	else
    	{
        	// Key quality and key signature
    		setKey(wd.getKeySignature(), wd.getKeyQuality());
    		
			// Note
			String cbDynamic = wd.getDynamic();
			String note = wd.getNote();
			String noteType = wd.getNoteType();
			chooseNote(note, noteType, cbDynamic);
			
			// Instrument
			String instrument = wd.getInstrument();
			chooseInstrument(instrument);
			
			// The smallest size jMusic can manage is a semiquaver/16th-note, and the MusicXML value for it is 4. 
			si.setDivisions(4);

			// Adding the phrase, part and score
			part.addPhrase(phrase);
			score.addPart(part);
			//View.notate(score);
    	}
    	
    	// Initialization of the GUI part
    	init();
		
		// Clef (2): it must be set AFTER init()
		setClef(clef);
		
		// Set score into ScoreInformation
		si.setScore(score);
	}
    
    // Called from GUI_Welcome, trying to open a .xml file
    public Notate(String clef)
    {
    	score = si.getScore();
    	init();
    	setClef(clef);
    }
    
    public Notate(Score score)
    {
    	this.score = score;
    	init();
    }
    

private void setKey(String keySignature, String keyQuality) {
		
	int keyNumber = Integer.parseInt(keySignature.substring(0,1));
	String keyAlter;
	
	if (keyNumber == 0){
		score.setKeySignature(keyNumber);
	} else {
		keyAlter = keySignature.substring(1,2);
		
		if (keyAlter.equals("#")){
			score.setKeySignature(keyNumber);
		} else if (keyAlter.equals("b")){
			score.setKeySignature(0-keyNumber);
		}
	}
	
	if (keyQuality.equals("Mayor")){
		score.setKeyQuality(0);
	} else if (keyQuality.equals("menor")){
		score.setKeyQuality(1);
	}
		
}

private void chooseNote(String pPitch, String pRhythmValue, String pDynamic){
	
	// Initial values
	int pitch = 60;
	double rhythmValue = 1.0;
	int dynamic = 0;
	
	// Choose the pitch
	if (pPitch.equals("Do"))
	{	pitch = 60;	}
	else if (pPitch.equals("Re"))
	{	pitch = 62;	}
	else if (pPitch.equals("Mi"))
	{	pitch = 64;	}
	else if (pPitch.equals("Fa"))
	{	pitch = 65;	}
	else if (pPitch.equals("Sol"))
	{	pitch = 67;	}
	else if (pPitch.equals("La"))
	{	pitch = 69;	}
	else if (pPitch.equals("Si"))
	{	pitch = 71;	}
	
	// Choose the duration
	if (pRhythmValue.equals("Corchea"))
	{	rhythmValue = 0.5;	}
	else if (pRhythmValue.equals("Negra"))
	{	rhythmValue = 1.0;	}
	else if (pRhythmValue.equals("Negra puntillo"))
	{	rhythmValue = 1.5;	}
	else if (pRhythmValue.equals("Blanca"))
	{	rhythmValue = 2.0;	}
	else if (pRhythmValue.equals("Blanca puntillo"))
	{	rhythmValue = 3.0;	}
	else if (pRhythmValue.equals("Redonda"))
	{	rhythmValue = 4.0;	}
	
	// Choose the dynamic
	if (pDynamic.equals("Silent"))
	{	dynamic = SILENT;	}
	else if (pDynamic.equals("PPP"))
	{	dynamic = PPP;	}
	else if (pDynamic.equals("PP"))
	{	dynamic = PP;	}
	else if (pDynamic.equals("P"))
	{	dynamic = P;	}
	else if (pDynamic.equals("MP"))
	{	dynamic = MP;	}
	else if (pDynamic.equals("MF"))
	{	dynamic = MF;	}
	else if (pDynamic.equals("F"))
	{	dynamic = F;	}
	else if (pDynamic.equals("FF"))
	{	dynamic = FF;	}
	else if (pDynamic.equals("FFF"))
	{	dynamic = FFF;	}

	phrase.add(new Note(pitch, rhythmValue, dynamic));
	/*
	double[] phrase2data = {};
	phrase.addNoteList(phrase2data);
	*/

}

//COMMIT CAMBIO

private void chooseInstrument(String instrument){
	if (instrument == "Violin") {

		phrase.setInstrument(40);

	}
	if (instrument == "Acordeon") {

		phrase.setInstrument(21);

	}
	if (instrument == "Piano") {

		phrase.setInstrument(0);

	}
	if (instrument == "Guitarra") {

		phrase.setInstrument(27);

	}
	if (instrument == "Flauta") {

		phrase.setInstrument(73);

	}

}


private void setClef(String clef) {
	
	if (clef.equals("FA"))
	{
		makeBassStave();
		si.setBassClef();
        for(int i=0; i<staveArray.length; i++) {
            staveArray[i].repaint();
        }
	}
	else if (clef.equals("SOL"))
	{
		makeTrebleStave();
		si.setTrebleClef();

        for(int i=0; i<staveArray.length; i++) {
            staveArray[i].repaint();
        }
	}
		
}

public void  add(Note note){
		phrase.addNote(note);
	}

public void addNoteList(Note[] notes) {
     for (int i = 0; i < notes.length; i++) {
         phrase.addNote(notes[i]);
     }
 }

    
    private void clearZoom() {
        zoomed = false;
    }        

    public Notate(Phrase phrase, int locX, int locY) {
        super("CPN: "+ phrase.getTitle());
        clearZoom();
        this.score = new Score(new Part(phrase));
        locationX = locX;
        locationY = locY;
        score = new Score(new Part(phrase));
        init();
    }
    
    public Notate(Score score, int locX, int locY) {
        super("CPN: "+ score.getTitle());
        clearZoom();
        this.score = score;
        locationX = locX;
        locationY = locY;
        init();
    }
    
    public void init() {
    	
        addWindowListener(this);
        // menus
        MenuBar menus = new MenuBar();
        Menu edit  = new Menu("Archivo", true);
        Menu features = new Menu("Herramientas", true);
        Menu player   = new Menu("Reproducción", true);
        //Menu view     = new Menu("Ver", true);
        
        //------
        /*
        newStave = new MenuItem("Nuevo pentagrama vacío");
        newStave.addActionListener(this);
        edit.add(newStave);
        */
        
        //------
        /*
        newWizard = new MenuItem("Nuevo pentagrama desde plantilla", new MenuShortcut(KeyEvent.VK_N));
        newWizard.addActionListener(this);
        edit.add(newWizard);
        */
        
        //edit.add("-");
        
        //-----
        impMXML = new MenuItem("Abrir MusicXML", new MenuShortcut(KeyEvent.VK_I));
        impMXML.addActionListener(this);
        edit.add(impMXML);
        
        //------
        expMXML = new MenuItem("Guardar (MusicXML)", new MenuShortcut(KeyEvent.VK_S));
        expMXML.addActionListener(this);
        edit.add(expMXML);
        
        expNewMXML = new MenuItem("Guardar como... (MusicXML)");
        expNewMXML.addActionListener(this);
        edit.add(expNewMXML);
        
        expLy = new MenuItem("Exportar PDF (requiere Lilypond)", new MenuShortcut(KeyEvent.VK_P));
        expLy.addActionListener(this);
        edit.add(expLy);

        edit.add("-");
        
        Menu importarMenu = new Menu("Importar");
        edit.add(importarMenu);
        
		// ------
		open = new MenuItem("Importar MIDI", new MenuShortcut(KeyEvent.VK_O));
		open.addActionListener(this);
		importarMenu.add(open);
		// ------
		openJmXml = new MenuItem("Importar jMusic XML");
		openJmXml.addActionListener(this);
		importarMenu.add(openJmXml);
		// ------
		openjm = new MenuItem("Importar archivo JM");
		openjm.addActionListener(this);
		importarMenu.add(openjm);

        //------
        edit.add("-");

        //--
        Menu exportarMenu = new Menu("Exportar");
        edit.add(exportarMenu);
      
        saveMidi = new MenuItem("Exportar MIDI");
        saveMidi.addActionListener(this);
        exportarMenu.add(saveMidi);
        //------
        saveJmXml = new MenuItem("Exportar jMusic XML");
        saveJmXml.addActionListener(this);
        exportarMenu.add(saveJmXml);
        //------
        saveJM = new MenuItem("Exportar archivo JM");
        saveJM.addActionListener(this);
        exportarMenu.add(saveJM);

        //------
        edit.add("-");
        
        //------
        welcome = new MenuItem("Volver a pantalla principal");
        welcome.addActionListener(this);
        edit.add(welcome);
        
        //------
        quit = new MenuItem("Cerrar", new MenuShortcut(KeyEvent.VK_Q));
        quit.addActionListener(this);
        edit.add(quit);
        
        // ---------- Herramientas (features) ---------- //
        addNotes = new MenuItem("Añadir nota mediante letra" );
        addNotes.addActionListener(this);
        features.add(addNotes);
      
        delete = new MenuItem("Borrar ultima nota", new MenuShortcut(KeyEvent.VK_D));
        delete.addActionListener(this);
        features.add(delete);
        //------
        clear = new MenuItem("Borrar todas las notas" , new MenuShortcut(KeyEvent.VK_C));
        clear.addActionListener(this);
        features.add(clear);
        //------
        features.add("-");
        //------
        keySig = new MenuItem("Mostrar/ocultar armadura", new MenuShortcut(KeyEvent.VK_K));
        keySig.addActionListener(this);
        features.add(keySig);
        //------
        timeSig = new MenuItem("Mostrar/ocultar compás", new MenuShortcut(KeyEvent.VK_T));
        timeSig.addActionListener(this);
        features.add(timeSig);
        //------
        barNumbers = new MenuItem("Mostrar/ocultar número de compás", new MenuShortcut(KeyEvent.VK_B));
        barNumbers.addActionListener(this);
        features.add(barNumbers);
        
        //------
        Menu staveMenu = new Menu("Pentagrama");
        features.add(staveMenu);
        
        trebleStave = new MenuItem("Clave Sol");
        trebleStave.addActionListener(this);
        staveMenu.add(trebleStave);
        
        bassStave = new MenuItem("Clave fa");
        bassStave.addActionListener(this);
        staveMenu.add(bassStave);
        
        /*
        pianoStave = new MenuItem("Clave piano");
        pianoStave.addActionListener(this);
        staveMenu.add(pianoStave);
        
        grandStave = new MenuItem("Grand Piano");
        grandStave.addActionListener(this);
        staveMenu.add(grandStave);
        
        automaticStave = new MenuItem("Automatico");
        automaticStave.addActionListener(this);
        staveMenu.add(automaticStave);
        */
        
        //------
        features.add("-");
        //------
        viewDetails = new MenuItem("Ver detalles técnicos");
        viewDetails.addActionListener(this);
        features.add(viewDetails);
        
        //------

        //-Features Added by Al C -----

        //------     
        /*  
        insertMidiFile = new MenuItem("Insert a MIDI file..." );
        insertMidiFile.addActionListener(this);
        features.add(insertMidiFile);

        appendMidiFile = new MenuItem("Append a MIDI file..." );
        appendMidiFile.addActionListener(this);
        features.add(appendMidiFile);
        */
        
        
        setParameters = new MenuItem("Editar parámetros" );
        setParameters.addActionListener(this);
        features.add(setParameters);
        
        //------        
        //------        

        //------        
//        adjustTiming = new MenuItem("Quantize Timing");
//        adjustTiming.addActionListener(this);
//        features.add(adjustTiming);

        //------        
        playAll = new MenuItem("Reproducir todo", new MenuShortcut(KeyEvent.VK_P));
        playAll.addActionListener(this);
        player.add(playAll);
        repeatAll = new MenuItem("Repetir todo" );
        repeatAll.addActionListener(this);
        player.add(repeatAll);

        //------        
        playMeasure = new MenuItem("Reproducir ultimo compás" );
        playMeasure.addActionListener(this);
        player.add(playMeasure);
        
        repeatMeasure = new MenuItem("Repetir último compás" );
        repeatMeasure.addActionListener(this);
        player.add(repeatMeasure);

        stopPlay = new MenuItem("Parar el sonido", new MenuShortcut(KeyEvent.VK_P, true) );
        stopPlay.addActionListener(this);
        player.add(stopPlay);
        
        //earTrain = new MenuItem("Ear Train" );
        //earTrain.addActionListener(this);
        //player.add(earTrain);

        
        /*
        viewZoom = new MenuItem("View phrase section", new MenuShortcut(KeyEvent.VK_V));
        viewZoom.addActionListener(this);
        view.add(viewZoom);
        */
        
        
        /*viewTitle = new MenuItem("Mostrar/ocultar título del pentagrama");
        viewTitle.addActionListener(this);
        view.add(viewTitle);*/

        //-------
        menus.add(edit);
        menus.add(features);
        menus.add(player);
        //menus.add(view);

        this.setMenuBar(menus);
        
        // components
        scroll = new ScrollPane(1);
       
        scroll.getHAdjustable().setUnitIncrement(10);
        scroll.getVAdjustable().setUnitIncrement(10);
                
        scoreBG = new Panel();
        layout = new GridBagLayout();
        scoreBG.setLayout(layout); //new GridLayout(score.size(), 1));
        constraints = new GridBagConstraints();
        setupConstraints();
        
        scroll.add(scoreBG);
        this.add(scroll); 
        
        setupArrays();
        makeAppropriateStaves();
               
        this.pack();
        this.setLocation(locationX, locationY);
       // calcHeight();
        //this.setSize(width, height + 40);
        /*
        // check window size against screen
        Toolkit tk = Toolkit.getDefaultToolkit();
        Dimension d = tk.getScreenSize();
        if(scroll.getSize().height > d.height) {
            System.out.println("Adjusting height");
            scroll.setSize(new Dimension(this.width, d.height));
        }
        */
        this.show();
    }
    
    
    
    private void setupArrays() {
        // set up arrays
        phraseArray = new Phrase[score.size()];
        staveArray = new Stave[score.size()];

        for (int i=0; i<staveArray.length; i++) {
            phraseArray[i] = score.getPart(i).getPhrase(0);
            staveArray[i] = new PianoStave();
            System.out.println("score key: " + score.getKeySignature());
            staveArray[i].setKeySignature(score.getKeySignature());
            System.out.println("stave key: " + staveArray[i].getKeySignature());
            System.out.println("score numerator: " + score.getNumerator());
            staveArray[i].setMetre(score.getNumerator());
            System.out.println("stave metre: " + staveArray[i].getMetre());
            staveArray[i].setBarNumbers(true);
        }
    }
    
    private void setupConstraints() {
        constraints.weightx = 100;
        constraints.weighty = 0;
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        //constraints.anchor = GridBagConstraints.WEST;
        constraints.fill = GridBagConstraints.BOTH;
    }
    
    private void calcHeight() {
        // work out the height
        height = 0;
        for (int i=0; i<staveArray.length; i++) {
            height += staveArray[i].getSize().height;
        }

    }
    
    private void makeAppropriateStaves(){
        Stave[] tempStaveArray  = new Stave[staveArray.length];
        for(int i=0; i<score.size(); i++) {
            Phrase currentPhrase = score.getPart(i).getPhrase(0);
            tempStaveArray[i] = new PianoStave();
            if(currentPhrase.getHighestPitch() < A6 &&
                currentPhrase.getLowestPitch() > FS3) tempStaveArray[i] = new TrebleStave();
            else if(currentPhrase.getHighestPitch() < F4 &&
                currentPhrase.getLowestPitch() > B1) tempStaveArray[i] = new BassStave();
            else if(currentPhrase.getHighestPitch() > A6 ||
                currentPhrase.getLowestPitch() < B1) tempStaveArray[i] = new GrandStave();
        }
        updateAllStaves(tempStaveArray);

    }   

    private void makeTrebleStave() {
        Stave[] tempStaveArray  = new Stave[score.size()];
        for(int i=0; i<staveArray.length; i++) {
             tempStaveArray[i] = new TrebleStave();
        }
        updateAllStaves(tempStaveArray);
    }
    
    private void updateAllStaves(Stave[] tempStaveArray) {
        int gridyVal = 0;
        int gridheightVal = 0;
        int totalHeight = 0;
        scoreBG.removeAll();
        for(int i=0; i<staveArray.length; i++) {
            // store current phrase parameters in new stave object
            tempStaveArray[i].setKeySignature(staveArray[i].getKeySignature());
            tempStaveArray[i].setMetre(staveArray[i].getMetre());
            tempStaveArray[i].setBarNumbers(staveArray[i].getBarNumbers());
            tempStaveArray[i].setPhrase(phraseArray[i]);
            // create new stave panel
            staveArray[i] = tempStaveArray[i];
            tempStaveArray[i] = null;
            // set and add constraints
            constraints.gridy = gridyVal;
            if(staveArray[i].getClass().isInstance(new TrebleStave()) || 
                staveArray[i].getClass().isInstance(new BassStave())) {
                    gridheightVal = 1;
            } else if(staveArray[i].getClass().isInstance(new PianoStave())) {
                gridheightVal = 2;
            } else {
                gridheightVal = 3;
            }
            constraints.gridheight = gridheightVal;
            // add to display
            scoreBG.add(staveArray[i], constraints);
            gridyVal += gridheightVal;
            totalHeight += staveArray[i].getPanelHeight();
        }
        //calcHeight();
        scroll.setSize(new Dimension(width, totalHeight));
        // check window size against screen
        Toolkit tk = Toolkit.getDefaultToolkit();
        tk.getScreenSize();
        this.setSize(new Dimension(700, 187));
        //this.setResizable(true);
        this.pack();
    }        

    private void makeBassStave() {
        Stave[] tempStaveArray  = new Stave[score.size()];
        for(int i=0; i<staveArray.length; i++) {
            tempStaveArray[i] = new BassStave();
        }
        updateAllStaves(tempStaveArray);
    }
        /*
            // store current phrase
            phrase = stave.getPhrase().copy();
            int tempKey = stave.getKeySignature();
            double tempTime = stave.getMetre();
            boolean tempBarNumbers = stave.getBarNumbers();
            // create new stave panel
            stave = new BassStave();
            scoreBG.removeAll();
            scoreBG.add(stave);
            scroll.setSize(width, stave.getSize().height + 20);
            this.pack();
            // replace stave
            stave.setPhrase(phrase);
            stave.setKeySignature(tempKey);
            stave.setMetre(tempTime);
            stave.setBarNumbers(tempBarNumbers);
    }
    */
    
    private void makePianoStave() {
        Stave[] tempStaveArray  = new Stave[score.size()];
        for(int i=0; i<tempStaveArray.length; i++) {
             tempStaveArray[i] = new PianoStave();
        }
        updateAllStaves(tempStaveArray);
    }
    
    /*
            // store current phrase
            phrase = stave.getPhrase().copy();
            int tempKey = stave.getKeySignature();
            double tempTime = stave.getMetre();
            boolean tempBarNumbers = stave.getBarNumbers();
            // create new stave panel
            stave = new PianoStave();
            scoreBG.removeAll();
            scoreBG.add(stave);
            scroll.setSize(width, stave.getSize().height + 20);
            this.pack();
            // replace stave
            stave.setPhrase(phrase);
            stave.setKeySignature(tempKey);
            stave.setMetre(tempTime);
            stave.setBarNumbers(tempBarNumbers);
    }    
    */
        
    private void makeGrandStave() {
        Stave[] tempStaveArray  = new Stave[score.size()];
        for(int i=0; i<tempStaveArray.length; i++) {
             tempStaveArray[i] = new GrandStave();
        }
        updateAllStaves(tempStaveArray);
    }
    
    /*
            // store current phrase
            phrase = stave.getPhrase().copy();
            int tempKey = stave.getKeySignature();
            double tempTime = stave.getMetre();
            boolean tempBarNumbers = stave.getBarNumbers();
            // create new stave panel
            stave = new GrandStave();
            scoreBG.removeAll();
            scoreBG.add(stave);
            scroll.setSize(width, stave.getSize().height + 20);
            this.pack();
            // replace stave
            stave.setPhrase(phrase);
            stave.setKeySignature(tempKey);
            stave.setMetre(tempTime);
            stave.setBarNumbers(tempBarNumbers);
    }       
    */ 

    class PlayRepeater extends Thread {
        
        JmMidiPlayer midiPlayer;
        Notate       n; 
        
        public PlayRepeater(         
                String       str,
                Notate       nParm,                
                JmMidiPlayer midiPlayerParm ) {
            super(str);           
            n         = nParm;         
            midiPlayer = midiPlayerParm;                    
        }                    
        
        public void run() {
            do {
                midiPlayer.play();                                                        
            } while(!n.timeToStop);                
        }
    }        
    
    public void actionPerformed(ActionEvent e){
        if(e.getSource() == close)
    	{
    		Object[] options = {"Guardar y cerrar",
    		                    "Sólo guardar",
    		                    "Cerrar sin guardar"};
    		int n = JOptionPane.showOptionDialog(frame,
    		    "¿Está seguro de querer cerrar la partitura?",
    		    "Cerrar sin partitura guardada",
    		    JOptionPane.YES_NO_CANCEL_OPTION,
    		    JOptionPane.QUESTION_MESSAGE,
    		    null,
    		    options,
    		    options[0]);
    		
    		if (n==0)
    		{
    			// Save and close
    			this.expJmToXml();
            	dispose();
    		}
    		else if (n==1)
    		{
    			// Only save
            	this.expJmToXml();
    		}
    		else if (n==2)
    		{
    			// Exit without saving
            	dispose();
    		}
    	}
        
        else if(e.getSource() == newStave) //new Notate(afterZoom);
        {
    		Object[] options = {"Guardar y crear nueva partitura",
                    "Sólo guardar",
                    "Nueva partitura sin guardar la antigua"};
			int n = JOptionPane.showOptionDialog(frame,
			    "¿Está seguro de querer crear una partitura vacía? Perderá los datos si no guarda la partitura actual.",
			    "Nueva partitura vacía",
			    JOptionPane.YES_NO_CANCEL_OPTION,
			    JOptionPane.QUESTION_MESSAGE,
			    null,
			    options,
			    options[1]);
			
			if (n==0)
			{
				// Save and close
				this.expJmToXml();
				showNewBlankStave();
			}
			else if (n==1)
			{
				// Only save
				this.expJmToXml();
			}
			else if (n==2)
			{
				// Exit without saving
				showNewBlankStave();
			}

        }
        else
        if(e.getSource() == open) openMidi();
        else
        if(e.getSource() == openjm) openJM();
        else
        if(e.getSource() == openJmXml) openJMXML();
        else
        if(e.getSource() == keySig) {
            for(int i=0; i<staveArray.length; i++) {
                if(staveArray[i].getKeySignature() == 0) {
                    staveArray[i].setKeySignature(2); //TODO interesante
                    staveArray[i].repaint();
                } else {
                    staveArray[i].setKeySignature(0);
                    staveArray[i].repaint();
                }
            }
        }
        else
        if(e.getSource() == timeSig) {
            for(int i=0; i<staveArray.length; i++) {
                if(staveArray[i].getMetre() == 0.0) {
                    staveArray[i].setMetre(4.0); //TODO interesante
                    staveArray[i].repaint();
                } else {
                    staveArray[i].setMetre(0.0);
                    staveArray[i].repaint();
                }
            }
        }
        else
        if(e.getSource() == saveJM) saveJM();
        else
        if(e.getSource() == saveJmXml) saveJMXML();
        else
        if(e.getSource() == saveMidi) saveMidi();
        else
        if(e.getSource() == quit)
    	{
    		Object[] options = {"Guardar y cerrar",
    		                    "Sólo guardar",
    		                    "Cerrar sin guardar"};
    		int n = JOptionPane.showOptionDialog(frame,
    		    "¿Está seguro de querer cerrar Kosmos?",
    		    "Cerrar Kosmos",
    		    JOptionPane.YES_NO_CANCEL_OPTION,
    		    JOptionPane.QUESTION_MESSAGE,
    		    null,
    		    options,
    		    options[0]);
    		
    		if (n==0)
    		{
    			// Save and close
    			this.expJmToXml();
    			System.exit(0);
    		}
    		else if (n==1)
    		{
    			// Only save
            	this.expJmToXml();
    		}
    		else if (n==2)
    		{
    			// Exit without saving
    			System.exit(0);
    		}
    	}
        else
        if(e.getSource() == delete) {
            this.deleteLastNote();
        } else
        if(e.getSource() == clear) {
            this.clear();
        }
        else
        if(e.getSource() == trebleStave ) {
            setCursor(new Cursor(Cursor.WAIT_CURSOR));
            makeTrebleStave();
            si.setTrebleClef();
            for(int i=0; i<staveArray.length; i++) {
                staveArray[i].repaint();
            }
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }
        else
        if(e.getSource() == bassStave) {
            setCursor(new Cursor(Cursor.WAIT_CURSOR));
            makeBassStave();
            si.setBassClef();
            for(int i=0; i<staveArray.length; i++) {
                staveArray[i].repaint();
            }
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }
        else
        if(e.getSource() == pianoStave) {
            setCursor(new Cursor(Cursor.WAIT_CURSOR));
            makePianoStave();
            for(int i=0; i<staveArray.length; i++) {
                staveArray[i].repaint();
            }
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }
        else
        if(e.getSource() == grandStave) {
            setCursor(new Cursor(Cursor.WAIT_CURSOR));
            makeGrandStave();
            for(int i=0; i<staveArray.length; i++) {
                staveArray[i].repaint();
            }
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }
        else
        if(e.getSource() == automaticStave) {
            setCursor(new Cursor(Cursor.WAIT_CURSOR));
            makeAppropriateStaves();
            for(int i=0; i<staveArray.length; i++) {
                staveArray[i].repaint();
            }
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }
        /*
        else
        if(e.getSource() == insertMidiFile) { }
        
            setCursor(new Cursor(Cursor.WAIT_CURSOR));
            Phrase savePhrase = phrase;
        	phrase = readMidiPhrase();
            phrase.addNoteList(
                savePhrase.getNoteList(),
                true
            );        
            stave.setPhrase(phrase);
            for(int i=0; i<staveArray.length; i++) {
                staveArray[i].repaint();
            }
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }
        
        else
        if(e.getSource() == appendMidiFile) { }
        
            setCursor(new Cursor(Cursor.WAIT_CURSOR));
            phrase.addNoteList(
                readMidiPhrase().getNoteList(),
                true
            );        
            stave.setPhrase(phrase);
            stave.repaint();
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }
        */
        else if(e.getSource() == setParameters) { // only works on the top stave
            ParmScreen parmScreen = new ParmScreen(this);
            staveArray[0].getPhrase().getTempo(); 					
                parmScreen.getParms(staveArray[0].getPhrase(), 15, 15 );				
            //}
            repaint();
        }
        else if( e.getSource() == playAll )  {
                Play.midi(score, false);   
        }
        else if  (e.getSource() == repeatAll) Play.midiCycle(score); 
        else if(e.getSource() == stopPlay) {
        	
        	Play.stopMidi(); // stop a single playback
                Play.stopMidiCycle(); // stop a cycle playback
        }         
        else if (e.getSource() == repeatMeasure) {
            Play.midiCycle(getLastMeasure());  
        }
        else if(e.getSource() == playMeasure ) {
            Play.midi(getLastMeasure(), false);   
        }
        
        else if(e.getSource() == addNotes) { // only works on the top stave
            LetterNotesEditor notesScreen 
                    = new LetterNotesEditor(this);
            notesScreen.getNotes(staveArray[0]);
            staveArray[0].repaint();
        }
        else if(e.getSource() == adjustTiming) { // only works on the top stave
            adjustTimeValues(staveArray[0].getPhrase());
            staveArray[0].repaint();
        }
        else if(e.getSource() == viewDetails) { // only works on the top stave
            PhraseViewer phraseViewer 
                    = new PhraseViewer(this);
            phraseViewer.showPhrase(
                            staveArray[0],
                            staveArray[0].getPhrase(),
                            15, 15 );               
        }
        else if(e.getSource() == viewZoom) { // only works on the top stave
            if (!zoomed) {
                CpnZoomScreen zoomSelector 
                        = new CpnZoomScreen(this);
                beforeZoom = staveArray[0].getPhrase().copy();                        
                afterZoom  = staveArray[0].getPhrase().copy();                        
                beforeZoom.empty();
                afterZoom.empty();
                zoomSelector.zoomIn(
                            beforeZoom,
                            staveArray[0].getPhrase(),
                            afterZoom);               
                if (beforeZoom.size() +
                    afterZoom.size() > 0 ) {
                    zoomed = true;
                    viewZoom.setLabel("View complete phrase");                        
                    repaint();
                }                                                    
            }
            else {
                CpnZoomScreen.zoomOut(
                            beforeZoom,
                            staveArray[0].getPhrase(),
                            afterZoom );               
                zoomed = false;
                viewZoom.setLabel("View phrase section");                        
                repaint();
            }                                                            
        }
        else if (e.getSource() == barNumbers) {
            for(int i=0; i<staveArray.length; i++ ) {
                staveArray[i].setBarNumbers(!staveArray[i].getBarNumbers());
                staveArray[i].repaint();
            }
        } 
        else if(e.getSource() == viewTitle){ 
        	toggleDisplayTitle();
        //else if (e.getSource() == earTrain ) {
        //    (new EarTrainer().show());
        //} 
        }
        
        else if (e.getSource() == expMXML) {
            this.expJmToXml();
        }
        else if (e.getSource() == expNewMXML) {
        	this.expNewJmToXml();
        }
        else if (e.getSource() == expLy) {
            si.setScore(score);
        	modules.ExpJmToLy.main();
        }
        else if (e.getSource() == impMXML) {
        	this.impXmlToJm();
        }
        else if (e.getSource() == newWizard) {
        	
        	Object[] options = {"Guardar y crear nueva partitura",
                    "Sólo guardar",
                    "Nueva partitura sin guardar la antigua"};
			int n = JOptionPane.showOptionDialog(frame,
			    "¿Está seguro de querer crear una partitura nueva desde plantilla? Perderá los datos si no guarda la partitura actual.",
			    "Nueva partitura desde plantilla",
			    JOptionPane.YES_NO_CANCEL_OPTION,
			    JOptionPane.QUESTION_MESSAGE,
			    null,
			    options,
			    options[1]);
			
			if (n==0)
			{
				// Save and close
				this.expJmToXml();
				showWizardWindow();
			}
			else if (n==1)
			{
				// Only save
				this.expJmToXml();
			}
			else if (n==2)
			{
				// Exit without saving
				showWizardWindow();
			}
        	
        }
        else if (e.getSource() == welcome)
        {
        	Object[] options = {"Guardar y volver al inicio",
                    "Sólo guardar",
                    "Volver al inicio sin guardar partitura"};
			int n = JOptionPane.showOptionDialog(frame,
			    "¿Está seguro de querer volver a la pantalla de inicio? Perderá los datos si no guarda la partitura actual.",
			    "Volver a pantalla de inicio",
			    JOptionPane.YES_NO_CANCEL_OPTION,
			    JOptionPane.QUESTION_MESSAGE,
			    null,
			    options,
			    options[1]);
			
			if (n==0)
			{
				// Save and go to welcome window
				this.expJmToXml();
				dispose();
				
				String[] args = null;
				gui.GUI_Welcome.main(args);
			}
			else if (n==1)
			{
				// Only save
				this.expJmToXml();
			}
			else if (n==2)
			{
				// Go to welcome window without saving
				dispose();
				
				String[] args = null;
				gui.GUI_Welcome.main(args);
			}
        }
                  
    }
    
    /**
    * Dialog to import a MIDI file
    */
     public void openMidi() {
        Score s = new Score();
        FileDialog loadMidi = new FileDialog(this, "Select a MIDI file.", FileDialog.LOAD);
        loadMidi.setDirectory( lastDirectory );
        loadMidi.setFile( lastFileName );
        loadMidi.show();
        String fileName = loadMidi.getFile();
        if (fileName != null) {
            lastFileName = fileName;
            lastDirectory = loadMidi.getDirectory();                        
            Read.midi(s, lastDirectory + fileName);  
            setNewScore(s);
        }
    }
    
    
    private void setNewScore(Score score) {
        this.score = score;
        // set up arrays
        setupArrays();        
        makeAppropriateStaves();
    }
    
    /**
     * Dialog to import a jm file
     */
     
     public void openJM() {
        FileDialog loadjm = new FileDialog(this, "Select a jm score file.", FileDialog.LOAD);
        loadjm.setDirectory( lastDirectory );
        loadjm.show();
        String fileName = loadjm.getFile();
        if (fileName != null) {
            Score s = new Score();
            lastDirectory = loadjm.getDirectory();  
            Read.jm(s, lastDirectory + fileName);
            setNewScore(s);
        }
    }
     
     public void impXmlToJm()
     {
    	JFrame alertFrame = new JFrame();
     	
     	try
     	{
     		String path = getImpXmlToJmPath();
     		
     		if (path != null)
				{
		            si.setPath(path);
		            modules.ImpXmlToJm.main();
		            setNewScore(si.getScore());
				}
				else 
				{
					// Window cancelled
					return;
				}
     	}
		catch (FileNotFoundException exc)
     	{
			exc.printStackTrace();
			JOptionPane.showMessageDialog(alertFrame, "Archivo no encontrado.");
		}
		catch (XMLStreamException exc)
		{
			exc.printStackTrace();
			JOptionPane.showMessageDialog(alertFrame, "Error en el formato del archivo. ¿Es un archivo XML?");
		}
		catch (StAXParsingError exc)
		{
			exc.printStackTrace();
			JOptionPane.showMessageDialog(alertFrame, "Error en la lectura del archivo. ¿Es un archivo de formato MusicXML?");
		}
     	/*catch (MusicXMLValidationError exc)
		{
			e.printStackTrace();
			
			JOptionPane.showMessageDialog(alertFrame, exc.getMessage());
		}*/
     }
     
     public void showNewBlankStave()
     {
    	clear();
     	score = new Score(new Part(staveArray[0].getPhrase()));
     	setNewScore(score);
     	si.setScore(score);
     	new Notate(si.getScore());
     	si.setPath(new String());
     	init();
     	dispose();
     }
     
     public void showWizardWindow()
     {
    	// TODO poner setPath a null?
     	clear();
     	score = new Score(new Part(staveArray[0].getPhrase()));
     	setNewScore(score);
     	si.setScore(score);
     	si.setPath(new String());
     	init();
     	dispose();

     	String[] args = null;
		gui.GUI_CreateScore.main(args);
     }
     
     public String getImpXmlToJmPath ()
     {

		String path = new String();

		FileFilter xmlType = new FileNameExtensionFilter("Archivo MusicXML (*.xml)", "xml");
		JFileChooser fc = new JFileChooser();
		fc.addChoosableFileFilter(xmlType);
		fc.setFileFilter(xmlType);

		int returnVal = fc.showOpenDialog(Notate.this);

		if (returnVal == JFileChooser.APPROVE_OPTION) {
			path = fc.getSelectedFile().getAbsolutePath();
		} else if (returnVal == JFileChooser.CANCEL_OPTION) {
			path = null;
		}

		return path;
     }
     
     public void expJmToXml()
     {
		if (si.getPath() == null)
		{
			expNewJmToXml();
		}
		else if (si.getPath().isEmpty())
		{
			expNewJmToXml();
		}
		else
		{
			si.setScore(score);
			modules.ExpJmToXml.main();
		}
    	 
     }
     
     public void expNewJmToXml()
     {
    	 getExpJmToXmlPath();
    	 
    	 if (!si.getPath().isEmpty())
    	 {
			expJmToXml();
    	 }
     }
     
     private void getExpJmToXmlPath() {
    	 
    	String path = new String();
    	 
 		// parent component of the dialog
		JFrame parentFrame = new JFrame();
		 
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setDialogTitle("Specify a file to save");   
		 
		int userSelection = fileChooser.showSaveDialog(parentFrame);
		 
		if (userSelection == JFileChooser.APPROVE_OPTION) {
			if(!fileChooser.getSelectedFile().getAbsolutePath().endsWith(".xml"))
			{
				
			    path = fileChooser.getSelectedFile().getAbsolutePath() + ".xml";
			}
			else 
			{
				path = fileChooser.getSelectedFile().getAbsolutePath();
			}
		}
		
		si.setPath(path);
 		
 	}
     
     
    
    /**
     * Dialog to import a jm XML file
     */
     
     public void openJMXML() {
        FileDialog loadjmxml = new FileDialog(this, "Select a jMusic XML score file.", FileDialog.LOAD);
        loadjmxml.setDirectory( lastDirectory );
        loadjmxml.show();
        String fileName = loadjmxml.getFile();
        if (fileName != null) {
            Score s = new Score();
            lastDirectory = loadjmxml.getDirectory(); 
            Read.xml(s, lastDirectory + fileName);
            setNewScore(s);
        }
    }
     
     
     
     
     // TODO borrar si no hay problemas
     
   //PARTE PARA HACER EXPORTAR A ARCHIVO MUSICXML 
     /*public void expJMtoXML() {
         FileDialog fd = new FileDialog(this, "Exportar como MusicXML",FileDialog.SAVE);
                 fd.show();
                             
         //write an XML file to disk
         if ( fd.getFile() != null) {
             Write.xml(score, fd.getDirectory()+fd.getFile());
             
         }
     }*/
     
     //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
     
     
     
     
     
     
     
    
    /**
     * Dialog to save phrase as a MIDI file.
     */
    public void saveMidi() {
        FileDialog fd = new FileDialog(this, "Save as a MIDI file...",FileDialog.SAVE);
                fd.show();
                            
        //write a MIDI file and stave properties to disk
        if ( fd.getFile() != null) {
            Write.midi(score, fd.getDirectory()+fd.getFile());
            /*
            for(int i=0; i<staveArray.length; i++){
                System.out.println(i);
                StavePhraseProperties props =
                    new StavePhraseProperties(
                            staveArray[i], staveArray[i].getPhrase());
                try {    
                    System.out.println("props");
                    props.writeToFile(                                         
                        fd.getDirectory()+fd.getFile());   
                }
                catch ( Throwable e) {
                    System.out.println(
                        "Error Saving Properties " +
                        e.getMessage() );                                                
                }
            } 
            */
                                              
        }
    }
    
    /**
     * Dialog to save score as a jMusic serialized jm file.
     */
    public void saveJM() {
        FileDialog fd = new FileDialog(this, "Save as a jm file...",FileDialog.SAVE);
                fd.show();
                            
        //write a MIDI file to disk
        if ( fd.getFile() != null) {
            Write.jm(score, fd.getDirectory()+fd.getFile());
        }
    }
    
    /**
     * Dialog to save score as a jMusic XML file.
     */
    public void saveJMXML() {
        FileDialog fd = new FileDialog(this, "Save as a jMusic XML file...",FileDialog.SAVE);
                fd.show();
                            
        //write an XML file to disk
        if ( fd.getFile() != null) {
            Write.xml(score, fd.getDirectory()+fd.getFile());
        }
    }

    
   
    /**
    * Get the first phrase from a MIDI file.
    */
    public Phrase readMidiPhrase() {
        FileDialog loadMidi = new FileDialog(this, "Select a MIDI file.", FileDialog.LOAD);
        loadMidi.show();
        String fileName = loadMidi.getFile();
        Phrase phr = new Phrase(0.0);
        Score scr = new Score();
        if (fileName != null) {
            Read.midi(scr, loadMidi.getDirectory() + fileName); 
        }
        scr.clean();
        if (scr.size() > 0 && scr.getPart(0).size() > 0) phr = scr.getPart(0).getPhrase(0);
        //System.out.println("Size = " + phr.size());
        return phr;
    }
    
    private Score getLastMeasure() {
        double beats = phraseArray[0].getNumerator();
        double endTime = score.getEndTime();
        int numbOfCompleteBars = (int)(endTime / beats);
        double startOflastBar = beats * numbOfCompleteBars;
        if (startOflastBar == endTime) startOflastBar -= beats;
        Score oneBar = score.copy(startOflastBar, endTime);
        
        for(int i=0; i<oneBar.size();i++){
            oneBar.getPart(i).getPhrase(0).setStartTime(0.0);
        }
        return oneBar;
    }
    
    private static double getRhythmAdjustment(
                        double  beats,
                        double  beatIncrement ) {
        double increments;
        increments = beats/beatIncrement;
        double tolerance;                                   
        tolerance = 0.00001;
        double answer;  
        answer = 0.0;
        double n;
        n = Math.floor(increments);
        while(( Math.floor(increments+tolerance) > n ) 
                && (tolerance > 0.00000000000001)) {
            answer = tolerance;
            tolerance = tolerance / 2;
        }                        
        return answer * beatIncrement;                            
    }                            
    
    private static void adjustTimeValues(Phrase phr) {
        int i;
        double t, dt, st;
        for( i = 0; i < phr.size(); ++i) {        
            t  = phr.getNote(i).getRhythmValue();
            dt = getRhythmAdjustment( t, 1.0 / 256.0 ); 
            phr.getNote(i).setRhythmValue(t+dt);
        }
        
        st = 0.0;
        for( i = 0; i < phr.size(); ++i) {        
            t  = phr.getNote(i).getRhythmValue();
            st = st + t;
            dt = getRhythmAdjustment( st, 1.0 ); 
            phr.getNote(i).setRhythmValue(t+dt);
            st = st + dt;
        }
    }  
    
    /**
    * Toggle the phrase title display
    */
    public void toggleDisplayTitle() {
    	System.out.println("Dentro de toggle display");

        for(int i=0; i<staveArray.length; i++) {
        	System.out.println("Dentro del array");

            staveArray[i].setDisplayTitle(!staveArray[i].getDisplayTitle());
        }
    }
    
    /**
     * Invoked when a window has been opened.
     */
    public void windowOpened(WindowEvent e) {
    }

    /**
     * Invoked when a window is in the process of being closed.
     * The close operation can be overridden at this point.
     */
    public void windowClosing(WindowEvent e) {
        if(e.getSource() == this) dispose();
        if(e.getSource() == keyDialog) keyDialog.dispose();
        if(e.getSource() == timeDialog) timeDialog.dispose();
    }

    

    /**
     * Invoked when a window has been closed.
     */
    public void windowClosed(WindowEvent e) {
    }

    /**
     * Invoked when a window is iconified.
     */
    public void windowIconified(WindowEvent e) {
    }

    /**
     * Invoked when a window is de-iconified.
     */
    public void windowDeiconified(WindowEvent e) {
    }

    /**
     * Invoked when a window is activated.
     */
    public void windowActivated(WindowEvent e) {
    }

    /**
     * Invoked when a window is de-activated.
     */
    public void windowDeactivated(WindowEvent e) {
    }

//	public Component getFrame() {
//		// TODO Auto-generated method stub
//		return null;
//	}

	public Window getFrame() {
		// TODO Auto-generated method stub
		return frame;
	}
	
	private void clear()
	{
		for(int i=0; i<staveArray.length; i++) {
	        staveArray[i].getPhrase().empty();
	        staveArray[i].repaint();
	    }
	}
	
	private void deleteLastNote()
	{
		for(int i=0; i<staveArray.length; i++) {
            staveArray[i].deleteLastNote();
        }
	}

}