package modules;

import gui.GUI_CreateScore;

import java.awt.Desktop;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import classes.FigureJm;
import classes.ScoreInformation;
import jm.JMC;
import jm.music.data.Note;
import jm.music.data.Part;
import jm.music.data.Phrase;
import jm.music.data.Score;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
//import java.net.URI;
import java.util.Scanner;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

public class ExpJmToLy extends Frame implements 
ActionListener, 
WindowListener, JMC {
	//

	static ScoreInformation si = ScoreInformation.getSI();
	static Score score = si.getScore();
	
	static Scanner keyboard;
	static String path;
	static FileWriter file = null;
	static PrintWriter writer = null;
	
	static int ind = 0; // indentation level
	
	static int measureNumber = 1;
	static String partId = "P1";

	public static void main()
	{
		try
		{

			Phrase phrase = score.getPart(0).getPhrase(0);
	 
			boolean chosen = getPath();
			
			if (chosen)
			{
				file = new FileWriter(path);
				writer = new PrintWriter(file);
		
				expHeaderToLy();
				writer.flush();
				
				expMeasureAttributesToLy();
				writer.flush();
				
				expFiguresToLy(phrase);
				writer.flush();
				
				expEndingToLy();
				writer.flush();
			}
			else
			{
				return;
			}
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		finally
		{
			try
			{
				if (file != null)
				{
					writer.close();
					file.close();
					
					// nativeDir = nativeDir.substring(0, nativeDir.lastIndexOf(File.separator));
					// String pathDir = path.substring(0, path.lastIndexOf(File.separator));
					File lyFile = new File(path);
					String lyPath = lyFile.getAbsolutePath();
					File lyDir = new File(lyPath).getParentFile();
					
					JFrame dialFrame = new JFrame();
					
					Object[] options = {"Sí, abrir carpeta", "No, continuar en Kosmos"};
					int n = JOptionPane.showOptionDialog(dialFrame,
							"Partitura extraida correctamente en " + path +
							"\n\nPara convertirlo a PDF deberá tener instalado Lilypond "
							+ "y hacer doble click sobre la partitura extraida.\n\n"
							+ "¿Desea abrir la carpeta?",
							"Extracción completa", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, 
							options, // the titles of buttons
							options[0]); // default button title
					
					if (n == JOptionPane.YES_OPTION)
					{
						if (Desktop.isDesktopSupported())
						{
							try
							{
						        Desktop.getDesktop().open(lyDir);
						    }
							catch (IOException e)
							{
								e.printStackTrace();
							}
						}
						else
						{
							JOptionPane.showMessageDialog(dialFrame, "Ha habido un problema al abrir la carpeta.");
						}
                    } 
				}
			}
			catch (Exception e2)
			{
				e2.printStackTrace();
			}
		}
	}

	private static boolean getPath()
	{
		boolean chosen = false;
		
		// parent component of the dialog
		JFrame parentFrame = new JFrame();
		 
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setDialogTitle("Elige dónde guardar la partitura en formato .ly");   
		 
		int userSelection = fileChooser.showSaveDialog(parentFrame);
		 
		if (userSelection == JFileChooser.APPROVE_OPTION) {
			if(!fileChooser.getSelectedFile().getAbsolutePath().endsWith(".ly"))
			{
			    path = fileChooser.getSelectedFile().getAbsolutePath() + ".ly";
			}
			else 
			{
				path = fileChooser.getSelectedFile().getAbsolutePath();
			}
			chosen = true;
		}
		
		return chosen;
		
	}
	
	/*
	 * idt: abreviation of "indentator". Indents with as many double-spaces as
	 * "ind" indicates
	 */
	private static String idt()
	{
		String indText = new String();
		
		if (ind >= 0)
		{
			indText = "";
			
			if (ind > 0)
			{
				for (int i=1; i <= ind; i++)
				{
					indText = indText + "  ";
				}
					
			}
		}
		
		return indText;
	}

	private static void expHeaderToLy()
	{
		writer.println(idt() + "% Converted from jMusic to Lilypond with Kosmos");
		
		/* Minimum version that can compile the .ly file.
		 * At the time of writing this comment, the latest 
		 * stable version of Lilypond is 2.18.2
		 */
		writer.println(idt() + "\\version \"2.18.2\"");
		writer.println(idt() + "");
		
		writer.println(idt() + "\\book {");
		writer.println(idt() + "");
		ind++;
		
			writer.println(idt() + "\\score {");
			writer.println(idt() + "");
			ind++;
			
				writer.println(idt() + "\\header {");
				ind++;
					writer.println(idt() + "encodingsoftware = \"Kosmos\"");
					writer.println(idt() + "title = \"" + score.getTitle() + "\"");
					writer.println(idt() + "composer = \"" + si.getComposer() + "\"");
					ind--;
				writer.println(idt() + "}");
				writer.println(idt() + "\\layout { }");
				writer.println(idt() + "\\midi {");
				ind++;
					writer.println(idt() + "\\tempo 4 = " + (int) score.getTempo());
					ind--;
				writer.println(idt() + "}");
				writer.println(idt() + "");
				
				writer.println(idt() + "\\new Staff {");
				ind++;
					writer.println(idt() + "\\new Voice {");
					ind++;
	}

	private static void expMeasureAttributesToLy()
	{
						// clef
						writer.println(idt() + "\\clef " + lyClef());
						// key
						writer.println(idt() + "\\key " + lyKey());
						// time
						writer.println(idt() + "\\time " + lyTime());
						
						writer.println(idt() + "");
	}
	
	private static String lyClef()
	{
		String clef = new String();
		
		if (si.getClef() == "SOL")
		{
			clef = "treble";
		}
		else
		{
			clef = "bass";
		}
		
		return clef;
	}
	
	private static String lyKey()
	{
		int keyS = score.getKeySignature();
		int keyQ = score.getKeyQuality();
		String key = new String();
		
		// KeyQuality==0 means Major, 1 means minor.
		if (keyS == 0)
		{
			if (keyQ == 0) { key = "c \\major"; }
			else { key = "a \\minor"; }
		}
		else if (keyS == 1)
		{
			if (keyQ == 0) { key = "g \\major"; }
			else { key = "e \\minor"; }
		}
		else if (keyS == 2)
		{
			if (keyQ == 0) { key = "d \\major"; }
			else { key = "b \\minor"; }
		}
		else if (keyS == 3)
		{
			if (keyQ == 0) { key = "a \\major"; }
			else { key = "fis \\minor"; }
		}
		else if (keyS == 4)
		{
			if (keyQ == 0) { key = "e \\major"; }
			else { key = "cis \\minor"; }
		}
		else if (keyS == 5)
		{
			if (keyQ == 0) { key = "b \\major"; }
			else { key = "gis \\minor"; }
		}
		else if (keyS == 6)
		{
			if (keyQ == 0) { key = "fis \\major"; }
			else { key = "dis \\minor"; }
		}
		else if (keyS == 7)
		{
			if (keyQ == 0) { key = "cis \\major"; }
			else { key = "ais \\minor"; }
		}
		else if (keyS == -1)
		{
			if (keyQ == 0) { key = "f \\major"; }
			else { key = "d \\minor"; }
		}
		else if (keyS == -2)
		{
			if (keyQ == 0) { key = "bes \\major"; }
			else { key = "g \\minor"; }
		}
		else if (keyS == -3)
		{
			if (keyQ == 0) { key = "ees \\major"; }
			else { key = "c \\minor"; }
		}
		else if (keyS == -4)
		{
			if (keyQ == 0) { key = "aes \\major"; }
			else { key = "f \\minor"; }
		}
		else if (keyS == -5)
		{
			if (keyQ == 0) { key = "des \\major"; }
			else { key = "bes \\minor"; }
		}
		else if (keyS == -6)
		{
			if (keyQ == 0) { key = "ges \\major"; }
			else { key = "ees \\minor"; }
		}
		else if (keyS == -7)
		{
			if (keyQ == 0) { key = "ces \\major"; }
			else { key = "aes \\minor"; }
		}
		
		return key;

	}
	
	private static String lyTime()
	{
		return score.getNumerator() + "/" + score.getDenominator();
	}

	private static void expFiguresToLy(Phrase phrase)
	{
		// List of notes
		Note[] noteArray = phrase.getNoteArray();
		// Conversion object for ever note
		FigureJm jm = null;
		
		// Position of the current note in "noteArray"
		int cont = 0;
		// Maximum of dividedDuration in a measure
		int maxDiv = score.getNumerator() * si.getDivisions();
		// Collected durations in current measure
		int collDiv = 0;
		// Duration waiting to be added from current note. If the note is not
		// tied, it will be equal to the full duration. If the note is tied, it
		// will show the remaining duration.
		int remDur = 0; 
		// Duration of the note that's about to be added. If the note were too
		// big for the remaining free space of the measure, it wouldn't fit, so
		// it should be pieced. The part that fits the space in the measure is
		// represented with "divDur".
		int divDur = 0;
		
		// First indentation before the figures
		writer.print(idt());

		while (noteArray.length > cont)
		{
			 
			Note n = noteArray[cont];
			jm = convJmToLy(n);
			remDur = calculateDivDur(n.getRhythmValue(), si.getDivisions());
			
			while (remDur!=0)
			{
				if (collDiv==0 && jm.isTieStart()){
					jm.setTieStop(true);
					jm.setTieStart(false);
				}
				
				if ((collDiv + remDur)>maxDiv)
				{
					divDur = maxDiv-collDiv;
				}
				else
				{
					divDur = remDur;
				}
				
				// Updates the duration if the figure is in the process of being divided
				updateDuration(jm, (double) divDur, si.getDivisions());
				
				collDiv = collDiv + divDur;
				
				if (collDiv == maxDiv)
				{
					if (!jm.isRest() && !(remDur==divDur))
					{
						jm.setTieStart(true);
					}
					
					printFigureTags(jm);
					printNewMeasure();
					
					collDiv=0;
				}
				else
				{
					printFigureTags(jm);
				}
				
				remDur = remDur - divDur;
				
			}
			
			cont++;

		 }
		
		// New line after all the notes
		writer.print("\n");

	}

	private static void updateDuration(FigureJm jm, Double divDur, int divisions)
	{
		jm.setDividedDuration(divDur.intValue());
		
		// Convert divided duration of the new figure to rhythmValue
		Double rhythmValue = ddToRv(divisions, divDur);
		jm.setRhythmValue(rhythmValue);
		
		// Update the duration
		parseDurationDot(rhythmValue, jm);
	}

	private static Double ddToRv(int divisions, Double divDur)
	{
		return divDur/divisions;
	}

	private static void expEndingToLy()
	{
						ind--;
					// Closing new Voice {
					writer.println(idt() + "}");
					ind--;
				// Closing new Staff {
				writer.println(idt() + "}");
				ind--;
			// Closing \score {
			writer.println(idt() + "}");
			ind--;
		// Closing \book {
		writer.println(idt() + "}");

	}
	
	private static void printNewMeasure()
	{
		// A measure error detection method similar to musicxml2ly's, per
		// each measure
		
		// | is a sign indicating that the measure has ended. It's not
		// compulsory for Lilypond, but it will help you to better find where
		// the mistake is.
		writer.print("| % " + measureNumber + "\n");
		
		// Measure counter updated
		measureNumber++;
		
		// Indentation
		writer.print(idt());
	}

	private static void printFigureTags(FigureJm jm)
	{
		/* Order:
		 * 1. Step/rest
		 * 2. Alter
		 * 3. Octave
		 * 4. Duration
		 * 5. Dot
		 * 6. Tie
		 */
		
		String result = new String();
		
		// 1. Step/rest
		if (jm.isRest())
		{
			result = result + "r";
		}
		else 
		{
			result = result + jm.getStep() + jm.getAlter() + jm.getOctave();
		}
		
		// 4. Duration
		result = result + jm.getDurationCrotchet();
		
		// 5. Dot
		if (jm.isDot())
		{
			result = result + ".";
		}
		
		// 6. Tie
		if (jm.isTieStart())
		{
			result = result + "~";
		}
		
		// Space at the ending
		result = result + " ";
		
		// Write the figure
		writer.print(result);
	}
	
	private static FigureJm convJmToLy(Note n)
	{
		/* Order:
		 * 1. Step/rest
		 * 2. Alter
		 * 3. Octave
		 * 4. Duration
		 * 5. Dot
		 * 6. Tie
		 */
		FigureJm fjm = new FigureJm();
		
		// Check whether it's a rest or not
		if (n.isRest() == true)
		{
			fjm.setRest(true);
		}
		else
		{
			fjm.setRest(false);
			
			// Set the step and alteration
			parseStepAlter(n, fjm);
			
			// Calculate octave
			parseOctave(n, fjm);
		}
		
		// Calculate duration and dot
		parseJmDurationDot(n, fjm);
		
		// The tie, if needed, will be added later on.
		
		return fjm;
		
	}
	
	private static void parseStepAlter(Note n, FigureJm jm)
	{
		String note = n.getNote();
		if (note.length() == 1)
		{
			 // If the note is natural, its name will only have 1 character
			 jm.setStep(n.getNote().toLowerCase());
			 jm.setAlter("");
		}
		else if (note.length() == 2)
		{
			 // If the note is not natural, the first character will be "step", and the second one "alter"
			 jm.setStep(note.substring(0,1).toLowerCase());
			 String alter = note.substring(1,2);
			 if (alter.equalsIgnoreCase("b"))
			 {
				 jm.setAlter("es");
			 }
			 else if (alter.equals("#"))
			 {
				 jm.setAlter("is");
			 }
		}
	}
	
	private static void parseOctave(Note n, FigureJm fjm)
	{
		String octave = new String();
		
		// Calculate octaves as if it were a MusicXML file
		int mXOct = n.getPitch();
		mXOct = mXOct / 12;
		mXOct = mXOct - 1;
		
		if (mXOct == 3)
		{
			octave = "";
		}
		else if (mXOct < 3)
		{
			for (int i=3; i > mXOct; i--)
			{
				octave = octave + ",";
			}
		}
		else if (mXOct > 3)
		{
			for (int i=3; i < mXOct; i++)
			{
				octave = octave + "\'";
			}
		}
		
		fjm.setOctave(octave);
	}
	
	private static void parseJmDurationDot(Note n, FigureJm fjm)
	{
		double rhyVal = n.getRhythmValue();
		
		parseDurationDot(rhyVal, fjm);
	}
	
	private static void parseDurationDot(double rhyVal, FigureJm fjm)
	{
		// The figure is dotted
		if ((1 % (rhyVal / 4)) != 0)
		{
			rhyVal = 1 / (rhyVal / 6);
			
			fjm.setDurationCrotchet((int) rhyVal);
			fjm.setDot(true);
		}
		else
		{
			rhyVal = 1 / (rhyVal / 4);
			
			fjm.setDurationCrotchet((int) rhyVal);
			fjm.setDot(false);
		}
	}
	
	private static int calculateDivDur(double rhythmValue, int divisions)
	{
		return (int) (rhythmValue * divisions);
	}

	private static String noteDuration(Double rhythmValue)
	{
		
		String duration = null;
		if (rhythmValue==0.25)
	 	{
			duration= "16th";
	 	}
		else if (rhythmValue==0.5)
	 	{
			duration= "eighth";
	 	}
	 	else if (rhythmValue==1.0)
	 	{
			duration= "quarter";
	 	}
	 	else if (rhythmValue==2.0)
	 	{
			duration= "half";
	 	}
	 	else if (rhythmValue==4.0)
	 	{
			duration= "whole";
	 	}
		return duration;
		
	}
	
	
	
	
	

	@Override
	public void windowActivated(WindowEvent arg0) {
		// TODO Auto-generated method stub

	}


	@Override
	public void windowClosed(WindowEvent arg0) {
		// TODO Auto-generated method stub

	}


	@Override
	public void windowClosing(WindowEvent arg0) {
		// TODO Auto-generated method stub

	}


	@Override
	public void windowDeactivated(WindowEvent arg0) {
		// TODO Auto-generated method stub

	}


	@Override
	public void windowDeiconified(WindowEvent arg0) {
		// TODO Auto-generated method stub

	}


	@Override
	public void windowIconified(WindowEvent arg0) {
		// TODO Auto-generated method stub

	}


	@Override
	public void windowOpened(WindowEvent arg0) {
		// TODO Auto-generated method stub

	}


	@Override
	public void actionPerformed(ActionEvent arg0) {
		// TODO Auto-generated method stub

	}

}
