package modules;

import java.io.FileNotFoundException;
import java.io.FileReader;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.Characters;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;

/*
import javax.swing.plaf.basic.BasicInternalFrameTitlePane.SystemMenuBar;
import javax.swing.text.html.parser.DTD;
import javax.xml.namespace.QName;
import javax.xml.stream.XMLEventReader;
import javax.xml.stream.XMLInputFactory;
import javax.xml.stream.XMLStreamConstants;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.events.Attribute;
import javax.xml.stream.events.Characters;
import javax.xml.stream.events.EndElement;
import javax.xml.stream.events.StartElement;
import javax.xml.stream.events.XMLEvent;
*/
//import com.sun.xml.internal.stream.events.CharacterEvent;

import classes.FigureJm;
import classes.ScoreInformation;
import classes.WizardData;
import exceptions.MusicXMLValidationError;
import exceptions.StAXParsingError;
import jm.music.data.Note;
import jm.music.data.Part;
import jm.music.data.Phrase;
import jm.music.data.Rest;
import jm.music.data.Score;

public class ImpXmlToJm
{

	private static ScoreInformation si = ScoreInformation.getSI();
	private static Score score;
	private static WizardData wd = new WizardData();

	private static XMLEventReader xer;

	public static WizardData main() throws FileNotFoundException, XMLStreamException, StAXParsingError
	{
		// Prepare the XML Input Factory
		XMLInputFactory factory = XMLInputFactory.newInstance();
		// StAX does not support DTD validating
		//factory.setProperty(XMLInputFactory.SUPPORT_DTD, true);
	    //factory.setProperty(XMLInputFactory.IS_NAMESPACE_AWARE, true);
	    //factory.setProperty(XMLInputFactory.IS_VALIDATING, true);
		
		xer = factory.createXMLEventReader(new FileReader(si.getPath()));

		score = new Score();

		// Check whether it's a MusicXML file - not working with StAX
		/*if (impMusicXMLCheck())
		{*/
			// Handle data in the header, such as work title, author, part,
			// ...
			impHeaderToJm();

			// Manager of the first measure's data, involving the key and
			// time signatures and related information
			impMeasureAttributesToJm();

			// Checks whether the tempo has been found
			if (wd.getTempo() == null)
			{
				impTempoToJm();
			}
			
			// Imports all the figures of the score
			impFiguresToJm();
		//}
			
		return wd;
		
	}

	private static boolean impMusicXMLCheck() throws XMLStreamException, MusicXMLValidationError, StAXParsingError
	{
		boolean check = false;

		if (xer.hasNext())
		{
			XMLEvent event;

			// <?xml version="1.0" encoding="UTF-8" standalone="no"?>
			event = xer.nextEvent();

			if (!event.isStartDocument())
			{
				throw new StAXParsingError("XML's StartDocument not found.");
			}

			/*
			 * <!DOCTYPE score-partwise PUBLIC
			 * "-//Recordare//DTD MusicXML 3.0 Partwise//EN"
			 * "http://www.musicxml.org/dtds/partwise.dtd">
			 */
			event = xer.nextEvent();

			if (event.getEventType() == javax.xml.stream.XMLStreamConstants.DTD)
			{
		        String text = event.toString();
		        
		        if (text.isEmpty())
				{
					throw new MusicXMLValidationError("MusicXML validation error. DTD is empty or missing.");
				}
				else if (!isValidMusicXMLFile(text))
				{
					throw new MusicXMLValidationError("MusicXML validation error. This is not a valid MusicXML file.");
				}
				else
				{
					check = true;
				}

			}

		}

		return check;
	}

	private static boolean isValidMusicXMLFile(String text)
	{

		/*
		 * <!DOCTYPE score-partwise PUBLIC
		 * "-//Recordare//DTD MusicXML 3.0 Partwise//EN"
		 * "http://www.musicxml.org/dtds/partwise.dtd">
		 */

		boolean valid = false;

		if (!text.contains("DOCTYPE") && !text.contains("MusicXML"))
		{
			valid = false;
		}
		else if (!text.contains("partwise") && !text.contains("timewise"))
		{
			valid = false;
		}
		else
		{
			valid = true;
		}

		return valid;
	}

	private static void impHeaderToJm() throws XMLStreamException, StAXParsingError
	{
		
		boolean found = false;
		XMLEvent event = null;
		String localPart = new String();
		
		// Find score-partwise
		while (xer.hasNext() && !found)
		{
			event = xer.nextEvent();

			if (event.isStartElement())
			{
				StartElement startElement = event.asStartElement();
				localPart = startElement.getName().getLocalPart();
				
				if (localPart.equals("score-partwise"))
				{
					found = true;
				}
				else if (localPart.equals("score-timewise"))
				{
					throw new StAXParsingError("Kosmos does not support MusicXML files with a <score-timewise> format.");
				}
			}
		}
		
		// Check whether there's a score-partwise tag
		if (!found)
		{
			throw new StAXParsingError("Invalid MusicXML structure: no part found.");
		}
		else
		{
			// Resetting the boolean to find a new tag
			found = false;
		}
		
		// Parse the other tags in header
		while (xer.hasNext() && !found)
		{
			event = xer.nextEvent();

			if (event.isStartElement())
			{
				StartElement se = event.asStartElement();
				localPart = se.getName().getLocalPart();
				
				if (localPart.equals("score-part"))
				{
					Attribute partId = se.getAttributeByName(new QName("id"));
					
					parseScorePart(partId);					
				}
				else if (localPart.equals("creator"))
				{
					event = xer.nextEvent();
					
					parseCreator(event);
				}
				else if (localPart.equals("work-title"))
				{
					event = xer.nextEvent();
					
					parseWorkTitle(event);
				}
				else if (localPart.equals("part"))
				{
					// Parsing of the header ends here
					found = true;
				}
			} 
		}

		si.setScore(score);
	}
	
	private static void parseScorePart(Attribute partId)
	{
		if (partId != null)
		{
			Part aPart = new Part(partId.getValue());
			score.add(aPart);
		}
	}

	private static void parseCreator(XMLEvent event)
	{
		String creator = new String();
		
		if (event.isCharacters())
		{
			// <creator type="composer">a</creator>
			Characters ce = event.asCharacters();
			creator = ce.getData();
		}
		
		//si.setComposer(creator);
		wd.setComposer(creator);
	}

	private static void parseWorkTitle(XMLEvent event)
	{
		String workTitle = new String();
		
		if (event.isCharacters())
		{
			// <work-title>t</work-title>
			Characters ce = event.asCharacters();
			workTitle = ce.getData();
		}
		
		//score.setTitle(workTitle);
		wd.setTitle(workTitle);
	}

	private static void impMeasureAttributesToJm() throws XMLStreamException
	{
		boolean found = false;
		XMLEvent event = null;
		String localPart = new String();
		
		String sign = new String();
		int line = 0;
		
		// Parse the tag <attributes>
		while (xer.hasNext() && !found)
		{
			event = xer.nextEvent();

			if (event.isStartElement())
			{
				StartElement se = event.asStartElement();
				localPart = se.getName().getLocalPart();
				
				if (localPart.equals("sound"))
				{
					Attribute tempo = se.getAttributeByName(new QName("tempo"));
					
					parseTempo(tempo);					
				}
				else if (localPart.equals("divisions"))
				{
					event = xer.nextEvent();
					
					parseDivisions(event);					
				}
				else if (localPart.equals("fifths"))
				{
					event = xer.nextEvent();
					
					parseFifths(event);
				}
				else if (localPart.equals("mode"))
				{
					event = xer.nextEvent();
					
					parseMode(event);
				}
				else if (localPart.equals("beats"))
				{
					event = xer.nextEvent();
					
					parseBeats(event);
				}
				else if (localPart.equals("beat-type"))
				{
					event = xer.nextEvent();
					
					parseBeatType(event);
				}
				else if (localPart.equals("sign"))
				{
					event = xer.nextEvent();
					
					sign = parseSign(event);
				}
				else if (localPart.equals("line"))
				{
					event = xer.nextEvent();
					
					line = parseLine(event);
				}
			}
			else if (event.isEndElement())
			{
				EndElement ee = event.asEndElement();
				localPart = ee.getName().getLocalPart();
				
				if (localPart.equals("attributes"))
				{
					parseClef(sign, line);
					
					// Parsing of the header ends here
					found = true;
				}
			}
		}
		
	}
	
	private static void parseClef(String sign, int line)
	{
		/*
			<clef>
				<sign>G</sign>
				<line>2</line>
		    </clef>
		 */
		
		// TODO create staff as treble/bass instead of "part"?
		if (sign.equals("G") && line == 2)
		{
			//si.setTrebleClef();
			wd.setClef("SOL");
			
		}
		else if (sign.equals("F") && line == 4)
		{
			//si.setBassClef();
			wd.setClef("FA");
		} 
	}
	
	private static void parseTempo(Attribute tempo)
	{
		//<sound tempo="80"/>
		if (tempo != null)
		{
			wd.setTempo(tempo.getValue());
		}
	}
	
	private static int parseLine(XMLEvent event)
	{
		// <line>2</line>
		Characters ce = event.asCharacters();
		return Integer.parseInt(ce.getData());
	}

	private static String parseSign(XMLEvent event)
	{
		// <sign>G</sign>
		Characters ce = event.asCharacters();
		return ce.getData();
	}

	private static void parseBeatType(XMLEvent event)
	{
		// <beats>4</beats>
		Characters ce = event.asCharacters();
		wd.setDenominator(ce.getData());
		//score.setDenominator(Integer.parseInt(ce.getData()));
	}

	private static void parseBeats(XMLEvent event)
	{
		// <beats>4</beats>
		Characters ce = event.asCharacters();
		wd.setNumerator(ce.getData());
		//score.setNumerator(Integer.parseInt(ce.getData()));
	}

	private static void parseFifths(XMLEvent event)
	{
		// <fifths>0</fifths>
		Characters ce = event.asCharacters();
		wd.setKeySignature(ce.getData());
		//score.setKeySignature(Integer.parseInt(ce.getData()));
	}
	
	private static void parseMode(XMLEvent event)
	{
		// <fifths>0</fifths>
		Characters ce = event.asCharacters();
		wd.setKeyQuality(ce.getData());
		//score.setKeySignature(Integer.parseInt(ce.getData()));
	}

	private static void parseDivisions(XMLEvent event)
	{
		// <divisions>4</divisions>
		Characters ce = event.asCharacters();
		si.setDivisions(Integer.parseInt(ce.getData()));
	}
	
	private static void impTempoToJm() throws XMLStreamException
	{
		boolean found = false;
		XMLEvent event = null;
		String localPart = new String();
		
		// Parse the tag <sound tempo="120">
		while (xer.hasNext() && !found)
		{
			event = xer.peek();
			System.out.println(event);

			if (event.isStartElement())
			{
				StartElement se = event.asStartElement();
				localPart = se.getName().getLocalPart();
				
				if (localPart.equals("sound"))
				{
					Attribute tempo = se.getAttributeByName(new QName("tempo"));
					
					parseTempo(tempo);					
				}
				else if (localPart.equals("note"))
				{
					// Parsing of the header ends here
					found = true;					
				}
			}
			
			if (!found)
			{
				xer.next();
			}
		}
		
	}

	private static void impFiguresToJm() throws XMLStreamException
	{
		boolean found = false;
		XMLEvent event = null;
		String localPart = new String();
		
		Phrase phrase = new Phrase();
		FigureJm figure = null;
		Note jm = null;
		boolean wasTieStart = false; // previous note's tieStart
		//boolean wasTieStop = false; // previous note's tieStop
		
		// Parsing of all the <note> tags
		while (xer.hasNext() && !found)
		{
			event = xer.nextEvent();

			if (event.isStartElement())
			{
				StartElement se = event.asStartElement();
				localPart = se.getName().getLocalPart();
				
				if (localPart.equals("tie"))
				{
					Attribute tie = se.getAttributeByName(new QName("type"));
					
					parseTie(figure, tie);
				}
				else if (localPart.equals("duration"))
				{
					event = xer.nextEvent();
					
					parseDuration(figure, event, wasTieStart);
				}
				else if (!wasTieStart)
				{
					/* That !wasTieStart if-clause will avoid
					 * the need to check 7 values that won't change
					 * their content.
					 */
					if (localPart.equals("note"))
					{
						/* If the previous note's tieStart is false,
						 * the current one's tieStop will also be false.
						 * Therefore, our current note will not be collecting
						 * data from previous tied notes.
						 */
							System.out.println("nota creada");
							figure = new FigureJm();
					}
					else if (localPart.equals("step"))
					{
						event = xer.nextEvent();
						
						parseStep(figure, event);
					}
					else if (localPart.equals("alter"))
					{
						event = xer.nextEvent();
						
						parseAlter(figure, event);
					}
					else if (localPart.equals("octave"))
					{
						event = xer.nextEvent();
						
						parseOctave(figure, event);
					}
					
					else if (localPart.equals("type"))
					{
						event = xer.nextEvent();
						
						parseType(figure, event);
					}
					else if (localPart.equals("rest"))
					{
						parseRest(figure);
					}
					else if (localPart.equals("dot"))
					{
						parseDot(figure);
					}
				}
			}
			else if (event.isEndElement())
			{
				EndElement ee = event.asEndElement();
				localPart = ee.getName().getLocalPart();

				if (localPart.equals("note"))
				{
					if (figure.isTieStart())
					{
						wasTieStart = true;
						figure.setTieStart(false);
						figure.setTieStop(false);
						// Go on to collect the next tied notes' durations
					}
					else
					{
						// Convert the note using all the collected data
						jm = convXmlToJm(figure);
						
						// Add figure to phrase
						phrase.add(jm);
						
						wasTieStart = false;
						
					}
				}
				else if (localPart.equals("part"))
				{
					// Parsing of the notes ends here
					found = true;
				}
			}
		}
		
		// Add phrase to part
		Part part = new Part(phrase);
		
		// Add part to score
		si.setScore(new Score(part));
	}

	private static Note convXmlToJm(FigureJm fjm) {
		
		Note f;
		int pitch = 0;
		Double rhythmValue = null;

		// Duration
		rhythmValue = ((double) fjm.getDividedDuration() / si.getDivisions()); 
		
		if (fjm.isRest())
		{
			f = new Rest(rhythmValue);
		}
		else
		{
			/*
			pitch = (Integer.valueOf(fjm.getOctave()) * 12)
					+ stepToExtraValue(fjm.getStep()) + 9;
			*/
			int octaves = Integer.valueOf(fjm.getOctave());
			octaves = (octaves + 1) * 12;
			
			int step = stepToExtraValue(fjm.getStep());
			
			int alter = 0;
			if (!fjm.getAlter().equals("0"))
			{
				alter = Integer.valueOf(fjm.getAlter());
			}
			
			pitch = octaves + step + alter;
			
			f = new Note(pitch, rhythmValue);
		}
		
		return f;
	}

	private static int stepToExtraValue(String step)
	{
		int extraValue = 0;
		
		if (step.equalsIgnoreCase("C"))
		{
			extraValue = 0;
		}
		else if (step.equalsIgnoreCase("D"))
		{
			extraValue = 2;
		}
		else if (step.equalsIgnoreCase("E"))
		{
			extraValue = 4;
		}
		else if (step.equalsIgnoreCase("F"))
		{
			extraValue = 5;
		}
		else if (step.equalsIgnoreCase("G"))
		{
			extraValue = 7;
		}
		else if (step.equalsIgnoreCase("A"))
		{
			extraValue = 9;
		}
		else if (step.equalsIgnoreCase("B"))
		{
			extraValue = 11;
		}
		  
		return extraValue;
	}

	private static void parseStep(FigureJm figure, XMLEvent event)
	{
		// <step>B</step>
		Characters ce = event.asCharacters();
		figure.setStep(ce.getData());
	}

	private static void parseAlter(FigureJm figure, XMLEvent event)
	{
		// <alter>1</alter>
		Characters ce = event.asCharacters();
		figure.setAlter(ce.getData());
	}

	private static void parseOctave(FigureJm figure, XMLEvent event)
	{
		// <octave>4</octave>
		Characters ce = event.asCharacters();
		figure.setOctave(ce.getData());
	}

	private static void parseDuration(FigureJm figure, XMLEvent event, boolean wasTieStart)
	{
		// <duration>4</duration>
		Characters ce = event.asCharacters();
		int divDur = Integer.parseInt(ce.getData());
		
		if (wasTieStart)
		{
			int oldDivDur = figure.getDividedDuration();
			divDur = divDur + oldDivDur;
		}
		
		figure.setDividedDuration(divDur);
	}

	private static void parseTie(FigureJm figure, Attribute tie)
	{
		// <tie type="start"/>
		if (tie.getValue().equals("start"))
		{
			figure.setTieStart(true);
		}
		else if (tie.getValue().equals("stop"))
		{
			figure.setTieStop(true);
		}
	}

	private static void parseType(FigureJm figure, XMLEvent event)
	{
		// <type>quarter</type>
		Characters ce = event.asCharacters();
		figure.setType(ce.getData());
	}

	private static void parseRest(FigureJm figure)
	{
		// <rest/>
		figure.setRest(true);
	}

	private static void parseDot(FigureJm figure)
	{
		// <dot/>
		figure.setDot(true);
	}
	
}
